package cn.com.qmth.markingaudit.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.markingaudit.bean.ExamDataQuery;
import cn.com.qmth.markingaudit.bean.ExamDataVo;
import cn.com.qmth.markingaudit.bean.MarkingScoreDataQuery;
import cn.com.qmth.markingaudit.bean.MarkingScoreDataVo;
import cn.com.qmth.markingaudit.bean.MarkingScoreInfo;
import cn.com.qmth.markingaudit.bean.QuestionDataQuery;
import cn.com.qmth.markingaudit.bean.QuestionDataVo;
import cn.com.qmth.markingaudit.bean.StudentDataQuery;
import cn.com.qmth.markingaudit.bean.StudentDataVo;
import cn.com.qmth.markingaudit.bean.SubjectDataQuery;
import cn.com.qmth.markingaudit.bean.SubjectDataVo;
import cn.com.qmth.markingaudit.dao.MonitorDao;
import cn.com.qmth.markingaudit.service.MonitorService;

@Service
public class MonitorServiceImpl implements MonitorService {

    @Autowired
    private MonitorDao monitorDao;

    @Override
    public List<ExamDataVo> examData(ExamDataQuery req) {
        List<ExamDataVo> ret = monitorDao.examData(req);
        if (CollectionUtils.isNotEmpty(ret)) {
            for (ExamDataVo vo : ret) {
                if (StringUtils.isNotBlank(vo.getSasConfig())) {
                    JSONObject js = JSONObject.parseObject(vo.getSasConfig());
                    if (js.containsKey("passScore")) {
                        vo.setPassScore(js.getInteger("passScore"));
                    }
                    if (js.containsKey("excellentScore")) {
                        vo.setExcellentScore(js.getInteger("excellentScore"));
                    }
                    vo.setSasConfig(null);
                }
            }
        }
        return ret;
    }

    @Override
    public List<StudentDataVo> studentData(StudentDataQuery req) {
        checkExamIdValid(req.getExamId(), req.getSchoolCode());
        return monitorDao.studentData(req);
    }

    private void checkExamIdValid(Integer examId, String schoolCode) {
        Integer id = monitorDao.getExamIdByIdAndSchoolCode(examId, schoolCode);
        if (id == null) {
            throw new StatusException("无效的考试id");
        }
    }

    @Override
    public List<SubjectDataVo> subjectData(SubjectDataQuery req) {
        checkExamIdValid(req.getExamId(), req.getSchoolCode());
        return monitorDao.subjectData(req);
    }

    @Override
    public List<QuestionDataVo> questionData(QuestionDataQuery req) {
        checkExamIdValid(req.getExamId(), req.getSchoolCode());
        return monitorDao.questionData(req);
    }

    @Override
    public List<MarkingScoreDataVo> markingScoreData(MarkingScoreDataQuery req) {
        checkExamIdValid(req.getExamId(), req.getSchoolCode());
        List<MarkingScoreDataVo> ret = monitorDao.markingScoreData(req);
        if (CollectionUtils.isNotEmpty(ret)) {
            List<Integer> ids = ret.stream().map(e -> e.getId()).collect(Collectors.toList());
            List<MarkingScoreInfo> infos = monitorDao.markingScoreInfo(ids);
            Map<Integer, List<MarkingScoreInfo>> map = new HashMap<>();
            for (MarkingScoreInfo info : infos) {
                List<MarkingScoreInfo> tem = map.get(info.getStudentId());
                if (tem == null) {
                    tem = new ArrayList<>();
                    map.put(info.getStudentId(), tem);
                }
                tem.add(info);
            }
            for (MarkingScoreDataVo vo : ret) {
                vo.setExamInfo(map.get(vo.getId()));
            }
        }
        return ret;
    }

}